@@ -21,13 +21,15 @@ public:
 
     std::error_code error;
     this->ro_mmap.map(path, error);
-    if (error) Rcpp::stop("Error when mapping file:\n  %s.\n", error.message());
+    if (error)
+      Rcpp::stop("Error when mapping file:\n  %s.\n", error.message());
 
     this->data = reinterpret_cast<const double*>(ro_mmap.data());
 
     this->compact = first_i.size() > 0;
   }
 
+  const double* i_x() { return reinterpret_cast<const double*>(ro_mmap.data()); }
   size_t nrow() const { return n; }
   size_t ncol() const { return m; }
   bool is_compact() const { return compact; }
