@@ -1,12 +1,10 @@
 /******************************************************************************/
 
 #include <bigsparser/SFBM.h>
+#include <bigsparser/SFBM-corr-compact.h>
 
 using namespace Rcpp;
 
-// Enable C++11 via this plugin (Rcpp 0.10.3 or later)
-// [[Rcpp::plugins(cpp11)]]
-
 /******************************************************************************/
 
 // [[Rcpp::export]]
@@ -15,8 +13,6 @@ NumericVector prodVec(Environment X, const NumericVector& y) {
   return sfbm->prod(y);
 }
 
-/******************************************************************************/
-
 // [[Rcpp::export]]
 NumericVector cprodVec(Environment X, const NumericVector& y) {
   XPtr<SFBM> sfbm = X["address"];
@@ -24,3 +20,17 @@ NumericVector cprodVec(Environment X, const NumericVector& y) {
 }
 
 /******************************************************************************/
+
+// [[Rcpp::export]]
+NumericVector corr_prodVec(Environment X, const NumericVector& y) {
+  XPtr<SFBM_corr_compact> sfbm = X["address"];
+  return sfbm->prod(y);
+}
+
+// [[Rcpp::export]]
+NumericVector corr_cprodVec(Environment X, const NumericVector& y) {
+  XPtr<SFBM_corr_compact> sfbm = X["address"];
+  return sfbm->cprod(y);
+}
+
+/******************************************************************************/
